/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl;

import java.io.IOException;

import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.NTCredentials;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScheme;
import org.apache.commons.httpclient.auth.CredentialsNotAvailableException;
import org.apache.commons.httpclient.auth.CredentialsProvider;
import org.apache.commons.httpclient.auth.NTLMScheme;
import org.apache.commons.httpclient.auth.RFC2617Scheme;
import org.apache.commons.httpclient.methods.ByteArrayRequestEntity;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.log4j.Logger;

import com.eviware.soapui.impl.wsdl.support.HttpClientSupport;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.model.iface.Response;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.model.iface.SubmitListener;

/**
 * Submit implementation for submitting a WsdlRequest
 * 
 * @author Ole.Matzura
 */

public class WsdlSubmit implements Runnable, Submit
{
   private final static Logger logger = Logger.getLogger( WsdlSubmit.class ); 
   private final WsdlRequest wsdlRequest;
   private PostMethod postMethod;
	private SubmitListener[] listeners;
	private String responseContent;
	private Status status;
	private Exception error;
	private WsdlResponse response = new WsdlResponse();
	private long timeTaken;
	private Thread thread;

   public WsdlSubmit( WsdlRequest wsdlRequest, SubmitListener [] listeners )
   {
      this.wsdlRequest = wsdlRequest;
		this.listeners = listeners;
		
		status = Status.CREATED;
   }
   
   public void submitRequest()
   {
		thread = new Thread( this );
		thread.start();
   }

   public void cancel()
   {
   	if( status == Status.CANCELED ) return;
      status = Status.CANCELED;
      
      logger.info( "Canceling request.." );
      if( postMethod.isRequestSent() )
      {
         postMethod.abort();
      }

      status = Status.CANCELED;
      
      for (int i = 0; i < listeners.length; i++) {
			listeners[i].afterSubmit( this );
		}
   }

   public void run()
   {
      try
      {
      	for (int i = 0; i < listeners.length; i++) {
   			if( !listeners[i].beforeSubmit( this )) return;
   		}

      	status = Status.RUNNING;
      	
         WsdlInterface wsdlInterface = (WsdlInterface) wsdlRequest.getOperation().getInterface();
         String strURL = wsdlInterface.getEndpointURL( wsdlRequest.getEndpoint() );

         postMethod = new PostMethod( strURL );
         postMethod.getParams().setParameter( CredentialsProvider.PROVIDER, new SoapUICredentialsProvider());
         postMethod.setDoAuthentication( true );
         
         // create new client since we dont want caching of authentications
         HttpClient httpclient = HttpClientSupport.getHttpClient();
         String encoding = wsdlRequest.getEncoding();
         if( encoding != null && encoding.trim().length() == 0) 
            encoding = null;
         if( encoding == null  )
            postMethod.setRequestHeader( "Content-type", "text/xml" );
         else
            postMethod.setRequestHeader( "Content-type", "text/xml;charset=" + encoding );
         
         String soapAction = wsdlRequest.getOperation().getAction();
         if( soapAction != null )
            postMethod.setRequestHeader( "SOAPAction", soapAction );

         // Request content will be retrieved directly
         // from the input stream
         String request = wsdlRequest.getRequestContent();
         if( request == null ) request = "";

         byte [] content = encoding == null ? request.getBytes() : request.getBytes( encoding );
         postMethod.setRequestEntity( new ByteArrayRequestEntity( content ));
         
         long now = System.currentTimeMillis();
         httpclient.executeMethod( new HostConfiguration(), postMethod, new HttpState() );
         timeTaken = System.currentTimeMillis() - now;
			if( status != Status.CANCELED )
			{         
	         byte[] responseBody = postMethod.getResponseBody();
	         responseContent = encoding == null ? new String( responseBody ) : 
				            new String( responseBody, encoding );
	         
	         status = Status.FINISHED;
			}
      }
      catch( Exception e1 )
      {
      	error = e1;
      	status = Status.ERROR;
      }
      finally
      {
      	if( postMethod != null )
      	{
      		postMethod.releaseConnection();
      	}

         if( status != Status.CANCELED )
         {
            for (int i = 0; i < listeners.length; i++) {
      			listeners[i].afterSubmit( this );
      		}
         }
      }
   }

	public Request getRequest() 
	{
		return wsdlRequest;
	}

	public String getResponseContent() {
		return responseContent;
	}

	public Status getStatus() {
		return status;
	}

	public Exception getError() {
		return error;
	}
	
	private class WsdlResponse implements Response
	{
		public String getContentAsString() {
			try 
			{
				return postMethod.getResponseBodyAsString();
			} 
			catch (IOException e) 
			{
				e.printStackTrace();
				return null;
			}
		}

		public long getContentLength() 
		{
			try {
				long length = postMethod.getResponseContentLength();
				if (length == -1)
					length = postMethod.getResponseBody().length;

				return length;
			} catch (Exception e) 
			{
				e.printStackTrace();
				return getContentAsString().length();
			}
		}

		public Request getRequest()
		{
			return wsdlRequest;
		}
		
	}

	public Response getResponse( boolean wait )
	{
		if( wait && (status == Status.RUNNING || status == Status.CREATED) )
		{
			synchronized (thread)
			{
				try
				{
					thread.wait();
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}
		
		return response;
	}

	public long getTimeTaken() {
		return timeTaken;
	}
	
	public class SoapUICredentialsProvider implements CredentialsProvider {

      public Credentials getCredentials(
          final AuthScheme authscheme, 
          final String host, 
          int port, 
          boolean proxy)
          throws CredentialsNotAvailableException 
      {
          if (authscheme == null) {
              return null;
          }
          try{
              if (authscheme instanceof NTLMScheme) {
                  logger.info(host + ":" + port + " requires Windows authentication");
                  return new NTCredentials(wsdlRequest.getUsername(), 
                  		wsdlRequest.getPassword(), host, wsdlRequest.getDomain());    
              } else
              if (authscheme instanceof RFC2617Scheme) {
            	  logger.info(host + ":" + port + " requires authentication with the realm '" 
                      + authscheme.getRealm() + "'");
                  return new UsernamePasswordCredentials(wsdlRequest.getUsername(), 
                  		wsdlRequest.getPassword());    
              } else {
                  throw new CredentialsNotAvailableException("Unsupported authentication scheme: " +
                      authscheme.getSchemeName());
              }
          } catch (IOException e) {
              throw new CredentialsNotAvailableException(e.getMessage(), e);
          }
      }
  }
}
