/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.iface;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.io.File;
import java.io.StringWriter;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JTabbedPane;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.syntax.jedit.JEditTextArea;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.support.SchemaUtils;
import com.eviware.soapui.model.DesktopPanel;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.support.JXmlTextArea;
import com.eviware.soapui.support.ProgressDialog;
import com.eviware.soapui.support.SwingWorker;
import com.eviware.soapui.support.XmlUtils;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * DesktopPanel for WsdlInterface. Loads all referenced wsdls/xsds for the specified WsdlInterface
 * and displays these in seperate tabs
 * 
 * @author Ole.Matzura
 */

public class WsdlInterfaceDesktopPanel extends JPanel implements DesktopPanel
{
   private final static Logger logger = Logger.getLogger( WsdlInterfaceDesktopPanel.class );
   private final WsdlInterface iface;
   private JTabbedPane tabbedPane;

   public WsdlInterfaceDesktopPanel(WsdlInterface iface)
   {
      super(new BorderLayout());

      this.iface = iface;

      tabbedPane = new JTabbedPane();
      tabbedPane.setTabPlacement( JTabbedPane.TOP );
      
      add( tabbedPane, BorderLayout.CENTER );
      String wsdlUrl = iface.getDefinition();
      
      Loader loader = new Loader( wsdlUrl );
      ProgressDialog progressDialog = SoapUI.getInstance() == null ? null : 
      	new ProgressDialog( "Progress", "Loading Definition..", 1, "Loading..", loader );
      
      progressDialog.setCancelLabel( "Run in background" );

      loader.start(progressDialog);
      if( progressDialog != null )
      	progressDialog.setVisible( true );
      
      setPreferredSize( new Dimension( 600, 500 ));
   }

	public ModelItem getModelItem()
	{
		return iface;
	}

	public boolean onClose()
	{
		return true;
	}

	public JComponent getComponent()
	{
		return this;
	}
	
	 private class Loader extends SwingWorker implements ProgressDialog.CancelHandler
   {
      private ProgressDialog progressDialog;
		private final String wsdlUrl;
		private JProgressBar progressBar;

      public Loader( String wsdlUrl )
      {
			this.wsdlUrl = wsdlUrl;
      }
      
      public void start(ProgressDialog progressDialog)
		{
			this.progressDialog = progressDialog;
			start();
		}

		public Object construct()
      {
      	try
         {
         	List<String> urls = SchemaUtils.getDefinitionUrls( wsdlUrl );
         	int tabCount = tabbedPane.getTabCount();
         	
            for (Iterator<String> iter = urls.iterator(); iter.hasNext();)
            {
   				addTab( iter.next() );
            }
            
            while( tabCount-- > 0 )
            	tabbedPane.remove( 0 );
            
            return null;
         }
         catch (Exception e)
         {
         	logger.error( "Failed to load WSDL; " + e.getClass().getSimpleName() + "; " + e.getMessage() ); 
            e.printStackTrace();
            return e;
         }
      }
      
      private void addTab(String url) throws Exception
   	{
      	int ix = url.startsWith( "file:" ) ? url.lastIndexOf( File.separatorChar ) : url.lastIndexOf( '/' );
      	String title = url.substring( ix+1);
      	
      	if( progressBar != null )
      		progressBar.setString( title );
      	else if( progressDialog != null )
      		progressDialog.setProgress( 1, title );
      	
      	XmlOptions options = new XmlOptions();
      	options.setLoadLineNumbers();
   		XmlObject xmlObject = XmlObject.Factory.parse( new URL( url ), options );
   		JPanel panel = new JPanel( new BorderLayout() );
   		JLabel label = new JLabel( url );
   		label.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
   		panel.add( label, BorderLayout.NORTH );
   		
   		JEditTextArea inputArea = new JXmlTextArea();
   		StringWriter writer = new StringWriter();
   		XmlUtils.serializePretty( xmlObject, writer );
         String xmlString = writer.toString();
   		inputArea.setText( xmlString );
         inputArea.setCaretPosition( 0 );
         inputArea.setEditable( false );
         
         panel.add( inputArea, BorderLayout.CENTER );
			tabbedPane.addTab( title, panel );
   	}

      public void finished()
      {
      	if( progressDialog != null )
      		progressDialog.setVisible( false );
      }

		public boolean onCancel()
		{
	
			progressBar = new JProgressBar(0, 1);
			progressBar.setSize( new Dimension( 120, 20 ));
		   progressBar.setStringPainted(true);
		   progressBar.setString("Loading Definition.." );
		   progressBar.setIndeterminate(true);
			
		   ButtonBarBuilder builder = ButtonBarBuilder.createLeftToRightBuilder();	
         builder.addGlue();
			builder.addFixed( progressBar );
         builder.addGlue();
         builder.setBorder( BorderFactory.createEmptyBorder( 10, 10, 10, 10 ));
         
			tabbedPane.addTab( "Loading.. ", builder.getPanel()  );
			return true;
		}
   }
}
