/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlObject;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.config.TransferValuesStepConfig;
import com.eviware.soapui.config.ValueTransferConfig;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.WsdlTestStep;
import com.eviware.soapui.impl.wsdl.panels.testcase.TransferResponseValuesTestStepPanelBuilder;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.support.XmlUtils;

/**
 * WsdlTestStep for transferring values from a WsdlTestRequest response to a 
 * WsdlTestRequest request using XPath expressions
 * 
 * @author Ole.Matzura
 */

public class TransferResponseValuesTestStep extends WsdlTestStep
{
	private TransferValuesStepConfig transferStepConfig;
	private TransferResponseValuesTestStepPanelBuilder panelBuilder;
	private final static Logger log = Logger.getLogger( TransferResponseValuesTestStep.class );
	private boolean canceled;

	public TransferResponseValuesTestStep(WsdlTestCase testCase, TestStepConfig config)
	{
		super(testCase, config);
	
      if( config.getConfig() != null )
      {
			transferStepConfig = (TransferValuesStepConfig) config.getConfig().changeType(TransferValuesStepConfig.type);
      }
      else
      {
         transferStepConfig = (TransferValuesStepConfig) config.addNewConfig().changeType( TransferValuesStepConfig.type );
      }
	}
	
	public TransferValuesStepConfig getTransferConfig()
	{
		return transferStepConfig;
	}
	
	protected void resetConfigOnMove(TestStepConfig config)
	{
		super.resetConfigOnMove(config);
		
		transferStepConfig = (TransferValuesStepConfig) config.getConfig().changeType(TransferValuesStepConfig.type);
	}

	public void run( TestRunner runner )
	{
		canceled = false;
		
		int ix = runner.getCurrentStepIndex();
		if( ix == 0 || ix == runner.getTestCase().getTestStepCount()-1 ) return;
		
		TestStep previousStep = runner.getTestCase().getTestStepAt( ix-1 );
		if( !(previousStep instanceof WsdlTestRequestStep )) return;

		TestStep nextStep = runner.getTestCase().getTestStepAt( ix+1 );
		if( !(nextStep instanceof WsdlTestRequestStep )) return;
		
		runTransfer( ((WsdlTestRequestStep) previousStep)
					.getTestRequest(), ((WsdlTestRequestStep) nextStep)
					.getTestRequest());
	}
	
	public void runTransfer( WsdlTestRequest sourceRequest, WsdlTestRequest targetRequest )
	{
		ValueTransferConfig[] transfers = transferStepConfig.getTransfersArray();
		for( int c = 0; c < transfers.length; c++ )
		{
			try
			{
				if( canceled ) 
					return;
				
				transferValues( sourceRequest, targetRequest, transfers[c]);
			}
			catch (Exception e)
			{
				log.error( e.getMessage() );
				e.printStackTrace();
			}
		}
	}

	public void transferValues(WsdlTestRequest sourceRequest, WsdlTestRequest targetRequest, ValueTransferConfig transfer) throws Exception
	{
		XmlObject previous = XmlObject.Factory.parse( sourceRequest.getResponseContent());
		XmlObject[] sourceValues = previous.selectPath( transfer.getSourcePath() );
		if( sourceValues.length == 0 ) 
		{
			log.info( "No match found for transfer sourcePath [" + transfer.getSourcePath() + 
					"] in transfer [" + transfer.getName() + "]");
			return;
		}

		XmlObject next = XmlObject.Factory.parse( targetRequest.getRequestContent());
	   XmlObject[] destValues = next.selectPath( transfer.getTargetPath() );
	   if( destValues.length == 0 ) 
	   {
			log.info( "No match found for transfer targetPath [" + transfer.getTargetPath() + 
					"] in transfer [" + transfer.getName() + "]" );
			return;
		}
	   
	   if( sourceValues.length != destValues.length )
	   	throw new Exception( "Number of matching values must be same in source response and destination request" +
	   			"; " + sourceValues.length + ":" + destValues.length );
	   
	   for( int c = 0; c < sourceValues.length; c++ )
	   {
	   	transferValue( sourceValues[c], destValues[c], transfer );
	   }
	   
	   if( canceled ) 
			return;
	   
	   targetRequest.setRequest( next.xmlText() );
	}

	private void transferValue(XmlObject source, XmlObject dest, ValueTransferConfig transfer) throws Exception
	{
		// just copy if nodes are of same type
		Node destNode = dest.getDomNode();
		Node sourceNode = source.getDomNode();
		short destNodeType = destNode.getNodeType();
		short sourceNodeType = sourceNode.getNodeType();
		
		// same type of node?
		if( destNodeType == sourceNodeType )
		{
			dest.set( source.copy() );
		}
		// text to attribute?
		else if( (sourceNodeType == Node.TEXT_NODE && destNodeType == Node.ATTRIBUTE_NODE) ||
				(sourceNodeType == Node.ATTRIBUTE_NODE && destNodeType == Node.TEXT_NODE) )
		{
			destNode.setNodeValue( sourceNode.getNodeValue() );
		}
		else if( sourceNodeType == Node.ELEMENT_NODE && 
				destNodeType == Node.ATTRIBUTE_NODE || destNodeType == Node.TEXT_NODE )
		{
			destNode.setNodeValue( XmlUtils.getElementText( (Element) sourceNode ));
		}
		else if( destNodeType == Node.ELEMENT_NODE && 
				sourceNodeType == Node.ATTRIBUTE_NODE || sourceNodeType == Node.TEXT_NODE )
		{
			// hmm.. not sure xmlbeans handles this ok
			XmlUtils.setElementText( (Element) destNode, sourceNode.getNodeValue() );
		}
	}

	public PanelBuilder getPanelBuilder()
	{
		if( panelBuilder == null )
			panelBuilder = new TransferResponseValuesTestStepPanelBuilder( this );
		
		return panelBuilder;
	}

	public boolean cancel()
	{
		canceled = true;
		return canceled;
	}
}
