/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.ImageIcon;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.eviware.soapui.config.CallConfig;
import com.eviware.soapui.config.RequestStepConfig;
import com.eviware.soapui.config.TestStepConfig;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.WsdlTestStep;
import com.eviware.soapui.impl.wsdl.teststeps.actions.CloneTestRequestStepAction;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Response;
import com.eviware.soapui.model.iface.Submit;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.support.InterfaceListenerAdapter;
import com.eviware.soapui.model.support.ProjectListenerAdapter;
import com.eviware.soapui.model.support.TestSuiteListenerAdapter;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.tree.AbstractTreeNode;
import com.eviware.soapui.model.tree.SoapUITreeNode;

/**
 * WsdlTestStep that executes a WsdlTestRequest
 * 
 * @author Ole.Matzura
 */

public class WsdlTestRequestStep extends WsdlTestStep implements PropertyChangeListener 
{
	private final static Log log = LogFactory.getLog( WsdlTestRequestStep.class );
   private RequestStepConfig requestStepConfig;
   private WsdlTestRequest testRequest;
	private WsdlOperation wsdlOperation;
	private Submit submit;
	private final InternalProjectListener internalProjectListener = new InternalProjectListener();
	private final InternalInterfaceListener internalInterfaceListener = new InternalInterfaceListener();
	private final InternalTestSuiteListener internalTestSuiteListener = new InternalTestSuiteListener();

   public WsdlTestRequestStep(WsdlTestCase testCase, TestStepConfig config)
   {
      super( testCase, config );
      
      testCase.getTestSuite().addTestSuiteListener( internalTestSuiteListener );
       
      if( config.getConfig() != null )
      {
			requestStepConfig = (RequestStepConfig) config.getConfig().changeType(RequestStepConfig.type);
         
         wsdlOperation = findWsdlOperation();
			if( wsdlOperation == null )
         {   
            log.error( "Could not find operation for test request" );
            requestStepConfig.setRequest(null);
         }
         else
         {
         	wsdlOperation.getInterface().getProject().addProjectListener( internalProjectListener );
         	wsdlOperation.getInterface().addInterfaceListener( internalInterfaceListener );
            testRequest = new WsdlTestRequest( wsdlOperation, requestStepConfig.getRequest(), this );
            testRequest.addPropertyChangeListener( this );
         }
      }
      else
      {
         requestStepConfig = (RequestStepConfig) config.addNewConfig().changeType( RequestStepConfig.type );
      }
      
      addAction( new CloneTestRequestStepAction( this ) );
   }
   
   public WsdlTestRequestStep(WsdlTestCase testCase, TestStepConfig testStep, WsdlRequest request)
	{
		this( testCase, testStep );

      requestStepConfig.setInterface( request.getOperation().getInterface().getName() );
      requestStepConfig.setOperation( request.getOperation().getName() );

      CallConfig testRequestConfig = requestStepConfig.getRequest();
      if( testRequestConfig == null ) testRequestConfig = requestStepConfig.addNewRequest();
      
      testRequestConfig.setName( request.getName() );
      testRequestConfig.setEncoding( request.getEncoding() );
      testRequestConfig.setEndpoint( request.getEndpoint() );
      testRequestConfig.setRequest( request.getRequestContent() );

      testRequest = new WsdlTestRequest( (WsdlOperation) request.getOperation(), testRequestConfig, this );
      testRequest.addPropertyChangeListener( this );
      
      wsdlOperation = findWsdlOperation();
      if( wsdlOperation == null )
      	throw new RuntimeException( "Failed to find operation for test request" );
      
      wsdlOperation.getInterface().addInterfaceListener( internalInterfaceListener );
      wsdlOperation.getInterface().getProject().addProjectListener( internalProjectListener );
	}

	private WsdlOperation findWsdlOperation()
   {
   	WsdlTestCase testCase = (WsdlTestCase) getTestCase();
      Project project = testCase.getTestSuite().getProject();
      WsdlOperation operation = null;
      for( int c = 0; c < project.getInterfaceCount(); c++ )
      {
         if( project.getInterfaceAt( c ).getName().equals( requestStepConfig.getInterface()))
         {
            WsdlInterface iface = (WsdlInterface) project.getInterfaceAt( c );
            for( int i = 0; i < iface.getOperationCount(); i++ )
            {
               if( iface.getOperationAt( i ).getName().equals( requestStepConfig.getOperation() ))
               {
                  operation = (WsdlOperation) iface.getOperationAt( i );
                  break;
               }
            }
            
            break;
         }
      }
      return operation;
   }

	protected void resetConfigOnMove(TestStepConfig config)
	{
		super.resetConfigOnMove(config);

		requestStepConfig = (RequestStepConfig) config.getConfig().changeType(RequestStepConfig.type);
		testRequest.updateConfig( requestStepConfig.getRequest() );
	}

	public ImageIcon getIcon()
   {
      return testRequest.getIcon();
   }

   public String getName()
   {
      return testRequest.getName();
   }
   
   public WsdlTestRequest getTestRequest()
   {
      return testRequest;
   }
   
	public void setName(String name)
	{
		testRequest.setName( name );
	}

	protected SoapUITreeNode createTreeNode()
   {
      return new TestRequestTreeNode();
   }
   
   private class TestRequestTreeNode extends AbstractTreeNode 
   {
      public TestRequestTreeNode()
      {
         super( WsdlTestRequestStep.this, getTestCase() );
         
         testRequest.addPropertyChangeListener( WsdlTestRequest.STATUS_PROPERTY, this );
      }
      
      public void release()
      {
      	super.release();
      	
      	testRequest.removePropertyChangeListener( this );
      }
   }

   public PanelBuilder getPanelBuilder()
   {
      return testRequest.getPanelBuilder();
   }

   public void propertyChange(PropertyChangeEvent arg0) 
   {
	  notifyPropertyChanged( arg0.getPropertyName(), arg0.getOldValue(), arg0.getNewValue());
   }

	public void run( TestRunner runner ) 
	{
   	submit = testRequest.submit();
		Response response = submit.getResponse( true );
		if( submit.getStatus() != Submit.Status.CANCELED )
			testRequest.setResponse( response.getContentAsString() );
		
		submit = null;
	}
	
	public WsdlAssertion getAssertionAt(int index)
	{
		return testRequest.getAssertionAt( index );
	}

	public int getAssertionCount()
	{
		return testRequest.getAssertionCount();
	}
	
	public class InternalProjectListener extends ProjectListenerAdapter 
	{
		public void interfaceRemoved(Interface iface)
		{
			if( wsdlOperation != null && wsdlOperation.getInterface() == iface )
			{
				log.debug( "Removing test step due to removed interface" );
				
				((WsdlTestCase)getTestCase()).removeTestStep( WsdlTestRequestStep.this );
			}
		}
	}

	public class InternalInterfaceListener extends InterfaceListenerAdapter
	{
		public void operationRemoved(Operation operation)
		{
			if( operation == wsdlOperation )
			{
				log.debug( "Removing test step due to removed operation" );
				
				((WsdlTestCase)getTestCase()).removeTestStep( WsdlTestRequestStep.this );
			}
		}
	}
	
	public boolean cancel()
	{
		if( submit != null )
			submit.cancel();
		
		return submit != null;
	}

	private class InternalTestSuiteListener extends TestSuiteListenerAdapter
	{
		public void testStepRemoved(TestStep testStep, int index)
		{
			if( testStep == WsdlTestRequestStep.this )
			{
				log.debug( "Releasing WsdlTestRequestStep" );
		      wsdlOperation.getInterface().removeInterfaceListener( internalInterfaceListener );
		      wsdlOperation.getInterface().getProject().removeProjectListener( internalProjectListener );
		      getTestCase().getTestSuite().removeTestSuiteListener( internalTestSuiteListener );
			}
		}
		
	}
}
