/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.assertions;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;

import org.apache.log4j.Logger;
import org.apache.xmlbeans.XmlAnySimpleType;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlObject;
import org.apache.xmlbeans.XmlOptions;
import org.custommonkey.xmlunit.XMLAssert;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.XmlUtils;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * Assertion that matches a specified XPath expression and its expected result against
 * the associated WsdlTestRequests response message
 * 
 * @author Ole.Matzura
 */

public class XPathContainsAssertion extends WsdlAssertion
{
	private final static Logger log = Logger.getLogger( XPathContainsAssertion.class );
	private String content;
   private String path;
	private JDialog configurationDialog;
	private JTextArea pathArea;
	private JTextArea contentArea;

   public XPathContainsAssertion(RequestAssertionConfig assertionConfig, WsdlTestRequest request)
   {
      super(assertionConfig, request);
      
      XmlObject[] paths = getConfiguration().selectPath( "$this/path" );
      if( paths.length == 1 ) 
      	path = paths[0].newCursor().getTextValue();
      
      paths = getConfiguration().selectPath( "$this/content" );
      if( paths.length == 1 ) 
      	content = paths[0].newCursor().getTextValue();
   }
   
   public String getContent()
	{
		return content;
	}

	public void setContent(String content)
	{
		this.content = content;
	}

	public String getPath()
	{
		return path;
	}

	public void setPath(String path)
	{
		this.path = path;
	}

	public String assertRequest(WsdlTestRequest request) throws AssertionException
   {
		return assertResponse( request.getResponseContent() );
   }
	
	public String assertResponse( String response ) throws AssertionException
	{
      try
		{
      	if( path == null ) return "Missing path for XPath assertion";
      	if( content == null ) return "Missing content for XPath assertion";
      	
      	XmlObject xml = XmlObject.Factory.parse( response );
      	XmlObject[] items = xml.selectPath( path );
      	
      	XmlObject contentObj = null;
      	
      	try
			{
				contentObj = XmlObject.Factory.parse(content);
			}
			catch (Exception e)
			{
				// this is ok.. it just means that the content to match is not xml but
				// (hopefully) just a string
			}
      	
      	if( items.length == 0 )
      		throw new Exception( "Missing content for xpath [" + path + "]" );
      	
      	XmlOptions options = new XmlOptions();
   		options.setSavePrettyPrint();
   		options.setSaveOuter();
   		
      	for( int c = 0; c < items.length; c++ )
      	{
      		try
				{
					if (contentObj == null)
					{
						if( items[c] instanceof XmlAnySimpleType )
						{
							XMLAssert.assertEquals(content, ((XmlAnySimpleType)items[c]).getStringValue() );
						}
						else
						{
							Node domNode = items[c].getDomNode();
							if (domNode.getNodeType() == Node.ELEMENT_NODE)
							{
								XMLAssert.assertEquals(content, XmlUtils.getElementText((Element) domNode));
							}
							else
							{
								XMLAssert.assertEquals(content, domNode.getNodeValue());
							}
						}
					}
					else
					{
						XMLAssert.assertXMLEqual(contentObj.xmlText(options), items[c].xmlText(options));
					}
					
					break;
				}
				catch (Throwable e)
				{
					if( c == items.length-1 )
						throw e;
				}
      	}
		}
		catch (Throwable e)
		{
			String msg = "XPathContains assertion failed for path [" + path + 
					"] : " + e.getClass().getSimpleName() + ":" + e.getMessage();
			
			throw new AssertionException( new AssertionError(msg) );
		}
      
      return "Response matches content for [" + path + "]";
   }

   public void configure()
   {
   	if( configurationDialog == null )
   		buildConfigurationDialog();
   	
   	pathArea.setText( path );
   	contentArea.setText( content );
   	
   	UISupport.showDialog( configurationDialog );
   }

   private void buildConfigurationDialog()
	{
      configurationDialog = new JDialog( SoapUI.getInstance().getFrame() );
		configurationDialog.setTitle("XPath Match configuration" );
      
      JPanel contentPanel = new JPanel( new BorderLayout() );
      JLabel label = new JLabel( "<html><b>Specify xpath expression and matching content below</b><br>" +
      		"declare namespaces with <code>declare namespace &lt;prefix&gt;='&lt;namespace&gt;';</code></html>" );
      label.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
      contentPanel.add( label, BorderLayout.NORTH );
      
      JSplitPane splitPane = UISupport.createVerticalSplit();
      pathArea = new JTextArea();
		splitPane.setTopComponent( new JScrollPane( pathArea ));
      
      contentArea = new JTextArea();
		splitPane.setBottomComponent( new JScrollPane( contentArea ));
		splitPane.setDividerLocation( 100 );
      splitPane.setBorder(BorderFactory.createEmptyBorder( 0, 3, 0, 3 ));
		
      contentPanel.add( splitPane, BorderLayout.CENTER );
      
      ButtonBarBuilder builder = new ButtonBarBuilder();
      
      builder.addFixed( new JButton( new SelectFromCurrentAction() ));
      builder.addRelatedGap();
      builder.addFixed( new JButton( new TestPathAction() ));
      builder.addRelatedGap();
      builder.addFixed( new JButton( new DeclareNamespacesFromCurrentAction()));
      
      builder.addGlue();
      builder.addFixed( new JButton( new OkAction() ));
      builder.addRelatedGap();
      builder.addFixed( new JButton( new CancelAction() ));
      builder.setBorder( BorderFactory.createEmptyBorder( 3, 3, 3, 3 ));
      
      contentPanel.add( builder.getPanel(), BorderLayout.SOUTH );
      
      configurationDialog.setContentPane( contentPanel );
      configurationDialog.setSize(500, 400);
      
      configurationDialog.setModal( false );
	}

	public XmlObject createConfiguration()
   {
      XmlObject config = XmlObject.Factory.newInstance();
      XmlCursor cursor = config.newCursor();
      cursor.toNextToken();
      cursor.insertElementWithText( "path", path );
      cursor.insertElementWithText( "content", content );
      cursor.dispose();
      return config;
   }

   public boolean isConfigurable()
   {
      return true;
   }
   
   public class OkAction extends AbstractAction
	{
		public OkAction()
		{
			super( "Save" );
		}

		public void actionPerformed(ActionEvent arg0)
		{
			setPath( pathArea.getText().trim() );
			setContent( contentArea.getText().trim() );
			setConfiguration( createConfiguration() );
			
			configurationDialog.setVisible( false );
		}
	}

   public class CancelAction extends AbstractAction
   {
   	public CancelAction()
   	{
   		super( "Cancel" );
   	}
   	
		public void actionPerformed(ActionEvent arg0)
		{
			configurationDialog.setVisible( false );
		}
   }
   
   public class DeclareNamespacesFromCurrentAction extends AbstractAction
   {
   	public DeclareNamespacesFromCurrentAction()
   	{
   		super( "Declare" );
   		putValue( Action.SHORT_DESCRIPTION, "Add namespace declaration from current response to xpath expression");
   	}
   	
		public void actionPerformed(ActionEvent arg0)
		{
			try
			{
				String responseContent = ((WsdlTestRequest) getRequest()).getResponseContent();
				if( responseContent == null || responseContent.trim().length() == 0 )
					return;
				
				pathArea.setText( XmlUtils.declareXPathNamespaces(responseContent ) + pathArea.getText() );
			}
			catch (Exception e)
			{
				log.error( e.getMessage() );
			}
	
		}
   }
   
   public class TestPathAction extends AbstractAction
	{
   	public TestPathAction()
   	{
   		super( "Test" );
   	}
   	
		public void actionPerformed(ActionEvent arg0)
		{
			String oldPath = getPath();
			String oldContent = getContent();
			
			setPath( pathArea.getText().trim() );
			setContent( contentArea.getText().trim() );
			
			try
			{
				String msg = assertRequest( ((WsdlTestRequest) getRequest()) );
				JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
						 msg, "Success", JOptionPane.INFORMATION_MESSAGE );
			}
			catch (AssertionException e)
			{
				JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
						 e.getMessage(), "Assertion Failed", JOptionPane.ERROR_MESSAGE );
			}

			setPath( oldPath );
			setContent( oldContent );
		}
	}
   
   public class SelectFromCurrentAction extends AbstractAction
	{
      private XmlOptions options;

		public SelectFromCurrentAction()
      {
      	super( "Select from current" );
      	
      	options = new XmlOptions();
			options.setSavePrettyPrint();
   		options.setSaveOuter();
   		options.setSaveAggressiveNamespaces();
      }
   	
		public void actionPerformed(ActionEvent arg0)
		{
			try
			{
				XmlObject xml = XmlObject.Factory
						.parse(((WsdlTestRequest) getRequest()).getResponseContent());
				
				String txt = pathArea.getSelectedText();
				if( txt == null ) txt = pathArea.getText();
				
				XmlObject[] items = xml.selectPath(txt.trim()); 
				
				contentArea.setText("");
				
				if( items.length == 0 )
				{
					JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
							"No match in current response", "Error", JOptionPane.ERROR_MESSAGE );
				}
				else if( items.length > 1 )
				{
					JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
							"More than one match in current response", "Error", JOptionPane.ERROR_MESSAGE );
				}
				else
				{
					if( items[0] instanceof XmlAnySimpleType )
					{
						contentArea.setText( ((XmlAnySimpleType)items[0]).getStringValue() );
					}
					else
					{
						Node domNode = items[0].getDomNode();
						if (domNode.getNodeType() == Node.ELEMENT_NODE)
						{
							contentArea.setText( items[0].xmlText(options));
						}
						else
						{
							contentArea.setText( domNode.getNodeValue());
						}
					}
				}
			}
			catch (Throwable e)
			{
				JOptionPane.showMessageDialog( SoapUI.getInstance().getFrame(), 
						 e.getMessage(), "Error", JOptionPane.OK_OPTION );
			}			
		}

	}
}
