/*
 *  soapui, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  SoapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of the GNU Lesser General Public License as published by the Free Software Foundation; 
 *  either version 2.1 of the License, or (at your option) any later version.
 *
 *  SoapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

import com.jgoodies.forms.layout.CellConstraints;
import com.jgoodies.forms.layout.FormLayout;
import com.jgoodies.forms.layout.RowSpec;

/**
 * Utility-class for creating forms
 */

public class SimpleForm
{
   private JPanel panel;
   private CellConstraints cc = new CellConstraints();
   private FormLayout layout;
   private RowSpec rowSpec;
   private int rowSpacing = 5;
   private Map<String,JComponent> components = new HashMap<String,JComponent>();
   private Map<JComboBox,Object[]> comboBoxMaps = new HashMap<JComboBox,Object[]>();
   private String rowAlignment = "top";
   private Map<String,String> hiddenValues;

   public SimpleForm()
   {
      this( new FormLayout( "10px,right:pref,10px,pref:grow,5px" ) );
   }

   public SimpleForm( FormLayout layout )
   {
      this.layout = layout;
      panel = new JPanel( layout );
      rowSpec = new RowSpec( rowAlignment + ":pref" );
   }

   public JPanel getPanel()
   {
      return panel;
   }

   public String getRowAlignment()
   {
      return rowAlignment;
   }

   public void setRowAlignment( String rowAlignment )
   {
      this.rowAlignment = rowAlignment;
   }

   public int getRowSpacing()
   {
      return rowSpacing;
   }

   public void setRowSpacing( int rowSpacing )
   {
      this.rowSpacing = rowSpacing;
   }

   public void addHiddenValue( String name, String value )
   {
      if( hiddenValues == null )
         hiddenValues = new HashMap<String,String>();

      hiddenValues.put( name, value );
   }

   public JButton addRightButton( Action action )
   {
      if( rowSpacing > 0 && !components.isEmpty() )
         addSpace( rowSpacing );

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      JButton button = new JButton( action );
      panel.add( button, cc.xy( 4, row, "right,bottom" ) );
      return button;
   }

   public void addSpace( int size )
   {
      if( size > 0 )
         layout.appendRow( new RowSpec( size + "px" ) );
   }

   public void addRightComponent( JComponent component )
   {
      if( rowSpacing > 0 && !components.isEmpty() )
         addSpace( rowSpacing );

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      panel.add( component, cc.xy( 4, row, "right,bottom" ) );
   }

   public JCheckBox appendCheckBox( String caption, String label, boolean selected )
   {
      JCheckBox checkBox = new JCheckBox( label, selected );
      components.put( caption, checkBox );
      append( caption, checkBox );
      return checkBox;
   }

   public void append( String label, JComponent component )
   {
      append( label, component, null );
   }

   public JComboBox appendComboBox( String label, Map values )
   {
      Object[] valueArray = new Object[values.size()];
      Object[] keyArray = new Object[values.size()];

      int ix = 0;
      for( Iterator i = values.keySet().iterator(); i.hasNext(); ix++ )
      {
         keyArray[ix] = i.next();
         valueArray[ix] = values.get( keyArray[ix] );
      }

      JComboBox comboBox = new JComboBox( valueArray );

      comboBoxMaps.put( comboBox, keyArray );

      append( label, comboBox );
      return comboBox;
   }

   public JComboBox appendComboBox( String label, String[] values )
   {
      JComboBox comboBox = new JComboBox( values );
      append( label, comboBox );
      return comboBox;
   }

   public void appendFixed( String label, JComponent component )
   {
      append( label, component, "left:pref" );
   }

   public void append( String label, JComponent component, String alignments )
   {
      if( rowSpacing > 0 && !components.isEmpty() )
         addSpace( rowSpacing );

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      if( label != null )
         panel.add( new JLabel( label ), cc.xy( 2, row ) );

      if( alignments == null )
         panel.add( component, cc.xy( 4, row ) );
      else
         panel.add( component, cc.xy( 4, row, alignments ) );

      components.put( label, component );
   }

   public void appendSeparator()
   {
      addSpace( rowSpacing );

      layout.appendRow( rowSpec );
      int row = layout.getRowCount();

      panel.add( new JSeparator(), cc.xywh( 2, row, 3, 1 ) );
   }

   public void appendTextField( String label )
   {
      append( label, new JTextField() );
   }

   public String getComponentValue( String label )
   {
      JComponent component = getComponent( label );
      if( component == null )
      {
         return (String) (hiddenValues == null ? null : hiddenValues.get( label ));
      }

      if( component instanceof JTextComponent )
      {
         return ((JTextComponent) component).getText();
      }

      if( component instanceof JComboBox )
      {
         JComboBox comboBox = ((JComboBox) component);
         if( comboBoxMaps.containsKey( component ) )
         {
            Object[] keys = (Object[]) comboBoxMaps.get( comboBox );
            int selectedItem = comboBox.getSelectedIndex();

            if( selectedItem != -1 ) return (String) keys[selectedItem];
         }
         else
            return comboBox.getSelectedItem().toString();
      }

      if( component instanceof JList )
      {
         return (String) ((JList) component).getSelectedValue();
      }

      if( component instanceof JCheckBox )
      {
         return String.valueOf( ((JCheckBox) component).isSelected() );
      }

      return null;
   }

   public JComponent getComponent( String label )
   {
      return (JComponent) components.get( label );
   }
}

